/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2012
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2012
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file DistributedDeadlockApi.h
 * 		P call definition for MUST distributed deadlock detection API calls.
 *
 * @author Tobias Hilbrich
 * @date 20.02.2012
 */

#ifndef DISTRIBUTED_DEADLOCK_API_H
#define DISTRIBUTED_DEADLOCK_API_H

#include "I_RequestTrack.h"
#include "I_CommTrack.h"

//== Function to pass a send to another DP2PMatch instance on the same layer
inline int PpassSendForMatching (
        MustParallelId pId,
        MustLocationId lId,
        int dest,
        int tag,
        MustCommType comm,
        MustDatatypeType type,
        int count,
        int mode)  {return 0;}

typedef int (*passSendForMatchingP) (
		MustParallelId pId,
		MustLocationId lId,
		int dest,
		int tag,
		MustCommType comm,
		MustDatatypeType type,
		int count,
		int mode,
		int toLevel /*IMPLICIT argument*/);

//== Function to pass an isend to another DP2PMatch instance on the same layer
inline int PpassIsendForMatching (
        MustParallelId pId,
        MustLocationId lId,
        int dest,
        int tag,
        MustCommType comm,
        MustDatatypeType type,
        int count,
        int mode,
        MustRequestType request)  {return 0;}

typedef int (*passIsendForMatchingP) (
        MustParallelId pId,
        MustLocationId lId,
        int dest,
        int tag,
        MustCommType comm,
        MustDatatypeType type,
        int count,
        int mode,
        MustRequestType request,
        int toLevel /*IMPLICIT argument*/);

//== Function to pass a start of a persistent send-request to another DP2PMatch instance on the same layer
inline int PpassSendStartForMatching (
        MustParallelId pId,
        MustLocationId lId,
        MustRequestType request)  {return 0;}

typedef int (*passSendStartForMatchingP) (
        MustParallelId pId,
        MustLocationId lId,
        MustRequestType request,
        int toLevel /*IMPLICIT argument*/);

//== Function to tell collective comm reductions whether any ancestor is tacking care of
//      checking complex MPI collective type matching with intra-layer communication
inline int PdCollMatchAncestorHasIntra (
        int ancestorHas)  {return 0;}

typedef int (*dCollMatchAncestorHasIntraP) (
        int ancestorHas);

//== Function to pass a collective operations type match info to another DCollectiveMatch instance on the same layer
inline int PpassTypeMatchInfo (
        MustParallelId pId,
        MustLocationId lId,
        MustRemoteIdType commRId,
        MustRemoteIdType typeRId,
        int numCounts,
        int* counts,
        int* forRanks,
        int collectiveNumber,
        int collId)  {return 0;}

typedef int (*passTypeMatchInfoP) (
        MustParallelId pId,
        MustLocationId lId,
        MustRemoteIdType commRId,
        MustRemoteIdType typeRId,
        int numCounts,
        int* counts,
        int* forRanks,
        int collectiveNumber,
        int collId,
        int toLevel /*IMPLICIT argument*/);

//== Function to pass a collective operations type match info to another DCollectiveMatch instance on the same layer.
//      Same as passTypeMatchInfo, but with multiple types (for MPI_Alltoallw).
inline int PpassTypeMatchInfoTypes (
        MustParallelId pId,
        MustLocationId lId,
        MustRemoteIdType commRId,
        int numCountsAndTypes,
        MustRemoteIdType* typeRIds,
        int* counts,
        int* forRanks,
        int collectiveNumber,
        int collId)  {return 0;}

typedef int (*passTypeMatchInfoTypesP) (
        MustParallelId pId,
        MustLocationId lId,
        MustRemoteIdType commRId,
        int numCountsAndTypes,
        MustRemoteIdType* typeRIds,
        int* counts,
        int* forRanks,
        int collectiveNumber,
        int collId,
        int toLevel /*IMPLICIT argument*/);

#endif /*DISTRIBUTED_DEADLOCK_API_H*/
