/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2012
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2012
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file I_MessageLogger.h
 *       @see I_MessageLogger.
 *
 *  @date 20.01.2011
 *  @author Tobias Hilbrich
 */

#include "I_Module.h"
#include "GtiEnums.h"
#include "I_CreateMessage.h"

#include "BaseIds.h"

#ifndef I_MESSAGELOGGER_H
#define I_MESSAGELOGGER_H

/**
 * Interface for logging messages.
 * Implementations may logg messages however they like.
 */
class I_MessageLogger : public gti::I_Module
{
public:
	/**
	 * Registers a new location.
	 * @param msgId unique identifier for this type of message, e.g. 7 for the "Buffer not allocated message".
	 * @param pId parallel Id.
	 * @param lId location id.
	 * @param msgType type of the message (error/warning/...) @see MustMessageType.
	 * @param text message text.
	 * @param textLen length of the text.
	 * @param numReferences number of extra location references used in the text.
	 * @param refPIds parallel ids of the extra locations, array of size numReferences.
	 * @param refLIds location ids of the extra locations, array of size numReferences.
	 * @return see gti::GTI_ANALYSIS_RETURN.
	 */
    virtual gti::GTI_ANALYSIS_RETURN log (
    		int msgId,
    		int hasLocation,
    		uint64_t pId,
    		uint64_t lId,
    		int msgType,
    		char *text,
    		int textLen,
    		int numReferences,
    		uint64_t* refPIds,
    		uint64_t* refLIds
    		) = 0;

}; /*class I_MessageLogger*/

#endif /*I_MESSAGELOGGER_H*/
